import { Route, Router } from "@solidjs/router";
import { listen } from "@tauri-apps/api/event";
import { createEffect, createSignal, For, onCleanup, Show } from "solid-js";
import { Login } from "./Login";
import { ProtectedRoute } from "./ProtectedRoute";
import { Search } from "./Search";
import { Settings } from "./Settings";
import { ImageViewer } from "./components/ImageViewer";
import type { PluginListener } from "@tauri-apps/api/core";
import {
    listenForShareEvents,
    type ShareEvent,
} from "tauri-plugin-sharetarget-api";
import { readFile } from "@tauri-apps/plugin-fs";
import { sendImage, sendImageFile } from "./network";

export const App = () => {
    const [logs, setLogs] = createSignal<string[]>([]);
    const [file, setFile] = createSignal<File>();

    createEffect(() => {
        // TODO: Don't use window.location.href
        const unlisten = listen("focus-search", () => {
            window.location.href = "/";
        });

        onCleanup(() => {
            unlisten.then((fn) => fn());
        });
    });

    createEffect(() => {
        let listener: PluginListener;
        const setupListener = async () => {
            listener = await listenForShareEvents(
                async (intent: ShareEvent) => {
                    const contents = await readFile(intent.stream).catch(
                        (error: Error) => {
                            console.warn("fetching shared content failed:");
                            throw error;
                        },
                    );
                    setFile(
                        new File([contents], intent.name ?? "no-name", {
                            type: intent.content_type,
                        }),
                    );
                    setLogs((l) => [...l, intent.uri]);
                },
            );
        };
        setupListener();
        return () => {
            listener?.unregister();
        };
    });

    createEffect(() => {
        const f = file();
        if (f == null) {
            return;
        }

        sendImageFile(f.name, f);
    });

    return (
        <>
            <ImageViewer />
            <p>Hello</p>
            <Show when={file()}>
                {(f) => <img alt="my-image" src={URL.createObjectURL(f())} />}
            </Show>
            <For each={logs()}>{(log) => <p>{log}</p>}</For>
            <Router>
                <Route path="/login" component={Login} />

                <Route path="/" component={ProtectedRoute}>
                    <Route path="/" component={Search} />
                    <Route path="/settings" component={Settings} />
                </Route>
            </Router>
        </>
    );
};
